package com.aliyun.vodplayer.demo;

import android.Manifest;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.TextView;

import com.aliyun.vodplayer.demo.activity.CopyRightActivity;
import com.aliyun.vodplayer.demo.activity.advance.DemeLiveAnswerUrlActivity;
import com.aliyun.vodplayer.demo.activity.advance.DemoUiPlayerActivity;
import com.aliyun.vodplayer.demo.activity.DemoDiagnosisiActivity;
import com.aliyun.vodplayer.demo.utils.BaseAppCompatActivity;
import com.aliyun.vodplayer.media.AliyunVodPlayer;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

public class DemoMainActivity extends BaseAppCompatActivity {
    //指定覆盖率文件存储路径
    public static String DEFAULT_COVERAGE_FILE_PATH = "/sdcard/coverage.ec";
    private String mCoverageFilePath;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_demo_main);

        findViewById(R.id.basic).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                int storagePermissionRet = ContextCompat.checkSelfPermission(DemoMainActivity.this, Manifest.permission.WRITE_EXTERNAL_STORAGE);
                if (storagePermissionRet != PackageManager.PERMISSION_GRANTED) {
                    requestPermission();
                    return;
                }
                Intent intent = new Intent();
                intent.setClass(DemoMainActivity.this, DemoBasePlayerActivity.class);
                startActivity(intent);
            }
        });
        findViewById(R.id.live_answer).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intent = new Intent();
                intent.setClass(DemoMainActivity.this, DemeLiveAnswerUrlActivity.class);
                startActivity(intent);
            }
        });
        findViewById(R.id.advance).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                int storagePermissionRet = ContextCompat.checkSelfPermission(DemoMainActivity.this, Manifest.permission.WRITE_EXTERNAL_STORAGE);
                if (storagePermissionRet != PackageManager.PERMISSION_GRANTED) {
                    requestPermission();
                    return;
                }
                Intent intent = new Intent();
                intent.setClass(DemoMainActivity.this, DemoAdvancePlayerActivity.class);
                startActivity(intent);
            }
        });

        findViewById(R.id.ui_player).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                int storagePermissionRet = ContextCompat.checkSelfPermission(DemoMainActivity.this, Manifest.permission.WRITE_EXTERNAL_STORAGE);
                if (storagePermissionRet != PackageManager.PERMISSION_GRANTED) {
                    requestPermission();
                    return;
                }

                Intent intent = new Intent();
                intent.setClass(DemoMainActivity.this, DemoUiPlayerActivity.class);
                //将vid和authinfo传递过去
                intent.putExtra("type", "skin");

                startActivity(intent);
            }
        });

        findViewById(R.id.web_diagnosis).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                int storagePermissionRet = ContextCompat.checkSelfPermission(DemoMainActivity.this, Manifest.permission.WRITE_EXTERNAL_STORAGE);
                if (storagePermissionRet != PackageManager.PERMISSION_GRANTED) {
                    requestPermission();
                    return;
                }
                Intent intent = new Intent();
                intent.setClass(DemoMainActivity.this, DemoDiagnosisiActivity.class);
                startActivity(intent);
            }
        });

        findViewById(R.id.copy_right).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                int storagePermissionRet = ContextCompat.checkSelfPermission(DemoMainActivity.this, Manifest.permission.WRITE_EXTERNAL_STORAGE);
                if (storagePermissionRet != PackageManager.PERMISSION_GRANTED) {
                    requestPermission();
                    return;
                }
                Intent intent = new Intent();
                intent.setClass(DemoMainActivity.this, CopyRightActivity.class);
                startActivity(intent);
            }
        });

        TextView version = (TextView) findViewById(R.id.version);

        version.setText("v" + AliyunVodPlayer.getSDKVersion());


        //创建覆盖率统计文件
        File file = new File(DEFAULT_COVERAGE_FILE_PATH);
        if(!file.exists()){
            try{
                file.createNewFile();
            }catch (IOException e){
                e.printStackTrace();}
        }

    }

    private int STORAGE_REQUEST_CODE = 100;

    private void requestPermission() {
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE)
                != PackageManager.PERMISSION_GRANTED) {
            // 向用户解释为什么需要这个权限
            new AlertDialog.Builder(this)
                    .setMessage(R.string.require_permision_msg)
                    .setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            //申请权限
                            ActivityCompat.requestPermissions(DemoMainActivity.this,
                                    new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, STORAGE_REQUEST_CODE);
                        }
                    })
                    .show();

        }
    }

    //获取覆盖率文件路径并生成结果
    private String getCoverageFilePath() {
        if (mCoverageFilePath == null) {
            return DEFAULT_COVERAGE_FILE_PATH;
        } else {
            return mCoverageFilePath;
        }
    }

    private void generateCoverageReport() {
        OutputStream out = null;
        try {
            out = new FileOutputStream(getCoverageFilePath(), false);
            Object agent = Class.forName("org.jacoco.agent.rt.RT")
                    .getMethod("getAgent")
                    .invoke(null);
            out.write((byte[]) agent.getClass().getMethod("getExecutionData", boolean.class)
                    .invoke(agent, false));
        } catch (Exception e) {
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }


    @Override
    protected void onDestroy() {

        generateCoverageReport();
        super.onDestroy();
    }
}
