package com.aliyun.vodplayer.demo.activity.advance;

import android.animation.ValueAnimator;
import android.content.DialogInterface;
import android.graphics.SurfaceTexture;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.util.Log;
import android.view.Surface;
import android.view.TextureView;
import android.view.View;
import android.view.ViewTreeObserver;
import android.view.WindowManager;
import android.widget.RelativeLayout;
import android.widget.Toast;

import com.alivc.player.AliVcMediaPlayer;
import com.alivc.player.AliyunErrorCode;
import com.alivc.player.MediaPlayer;
import com.alivc.player.VcPlayerLog;
import com.aliyun.vodplayer.demo.R;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.LiveAnswerView;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.bean.AnswerInfo;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.bean.AnswerRequestInfo;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.bean.QuestionInfo;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.request.AnswerRequest;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.request.GetQuestionReportRequest;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.request.GetQuestionRequest;
import com.aliyun.vodplayer.demo.activity.basic.LiveModeActivity;
import com.aliyun.vodplayer.demo.utils.BaseAppCompatActivity;
import com.aliyun.vodplayer.utils.BaseRequest;
import com.aliyun.vodplayer.utils.JsonUtil;
import com.aliyun.vodplayerview.utils.NetWatchdog;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.ref.WeakReference;
import java.util.UUID;

public class DemoLiveAnswerActivity extends BaseAppCompatActivity {

    private LiveAnswerView mLiveAnswerView;

    private ValueAnimator maxSurfaceAnim;
    private ValueAnimator minSurfaceAnim;

    private String liveUrl = null;
    private String liveId = null;

    private Surface videoSurface;

    private NetWatchdog netWatchdog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);


        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

        setContentView(R.layout.activity_demo_live_answer);

        initNetWatchDog();
        initLivePlayer();
        initSurfaceAnim();

        mLiveAnswerView = (LiveAnswerView) findViewById(R.id.live_answer_view);
        mLiveAnswerView.setOnTimeRemainListener(new LiveAnswerView.OnTimeRemainListener() {
            @Override
            public void decountEnd() {
                //不答题的情况下
                if (mLiveAnswerView.getMyAnswer() == null) {
                    canAnswer = false;
                }

                mLiveAnswerView.hide();
                maxLiveSurface();
            }

            @Override
            public void resultEnd() {
                mLiveAnswerView.hide();
                maxLiveSurface();
            }

            @Override
            public void onChoose(AnswerRequestInfo answerRequestInfo) {
                sendMyAnswerToServer(answerRequestInfo);
            }
        });
    }

    private void initNetWatchDog() {
        netWatchdog = new NetWatchdog(this);
        netWatchdog.setNetChangeListener(new NetWatchdog.NetChangeListener() {
            @Override
            public void onWifiTo4G() {

            }

            @Override
            public void on4GToWifi() {
                if(aliVcMediaPlayer !=null) {
                    VcPlayerLog.d("lfj0124", " to wifi .. aliVcMediaPlayer.islaying = " + aliVcMediaPlayer.isPlaying());
                    if(!aliVcMediaPlayer.isPlaying()){
                        aliVcMediaPlayer.stop();
                        aliVcMediaPlayer.prepareAndPlay(liveUrl);
                    }
                }
            }

            @Override
            public void onNetDisconnected() {
            }
        });
        netWatchdog.startWatch();
    }


    private void initSurfaceAnim() {

        maxSurfaceAnim = ValueAnimator.ofFloat(0, 1);
        maxSurfaceAnim.setDuration(400);
        maxSurfaceAnim.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
            @Override
            public void onAnimationUpdate(ValueAnimator animation) {
                float value = (float) animation.getAnimatedValue();
                RelativeLayout.LayoutParams layoutParams = (RelativeLayout.LayoutParams) surfaceView.getLayoutParams();

                layoutParams.width = (int) ((maxSurfaceWith - minSurfaceWith) * value + minSurfaceWith);
                layoutParams.height = (int) ((maxSurfaceHeight - minSurfaceHeight) * value + minSurfaceHeight);

                layoutParams.bottomMargin = (int) ((0 - minSurfaceMagin) * value + minSurfaceMagin);
                layoutParams.rightMargin = (int) ((0 - minSurfaceMagin) * value + minSurfaceMagin);

                Log.d("lfj0117", "maxSurfaceAnim value = " + value + " , width = " + layoutParams.width + " , margin = " + layoutParams.bottomMargin);

                surfaceView.setLayoutParams(layoutParams);
            }
        });

        minSurfaceAnim = ValueAnimator.ofFloat(0, 1);
        minSurfaceAnim.setDuration(400);
        minSurfaceAnim.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
            @Override
            public void onAnimationUpdate(ValueAnimator animation) {
                float value = (float) animation.getAnimatedValue();
                RelativeLayout.LayoutParams layoutParams = (RelativeLayout.LayoutParams) surfaceView.getLayoutParams();

                layoutParams.width = (int) ((minSurfaceWith - maxSurfaceWith) * value + maxSurfaceWith);
                layoutParams.height = (int) ((minSurfaceHeight - maxSurfaceHeight) * value + maxSurfaceHeight);

                layoutParams.bottomMargin = (int) ((minSurfaceMagin - 0) * value + 0);
                layoutParams.rightMargin = (int) ((minSurfaceMagin - 0) * value + 0);

                Log.d("lfj0117", "minSurfaceAnim value = " + value + " , width = " + layoutParams.width + " , margin = " + layoutParams.bottomMargin);

                surfaceView.setLayoutParams(layoutParams);
            }
        });
    }

    private AliVcMediaPlayer aliVcMediaPlayer;
    private TextureView surfaceView;

    private int maxSurfaceWith = -1;
    private int maxSurfaceHeight = -1;
    private int minSurfaceWith = -1;
    private int minSurfaceHeight = -1;
    private int minSurfaceMagin = -1;

    private void initLivePlayer() {
        liveUrl = getIntent().getStringExtra("url");
        int index = liveUrl.indexOf("//");
        liveId = liveUrl.substring(index + 2);

        if (liveId.endsWith(".flv")) {
            //去掉flv的结尾
            liveId = liveId.substring(0, liveId.length() - 4);
        }

        surfaceView = (TextureView) findViewById(R.id.surfaceView);


        surfaceView.getViewTreeObserver().addOnGlobalLayoutListener(new ViewTreeObserver.OnGlobalLayoutListener() {
            @Override
            public void onGlobalLayout() {
                if (maxSurfaceWith < 0) {

                    maxSurfaceWith = surfaceView.getWidth();
                    maxSurfaceHeight = surfaceView.getHeight();

                    minSurfaceWith = maxSurfaceWith / 4;
                    minSurfaceHeight = maxSurfaceHeight / 4;
                    minSurfaceMagin = maxSurfaceWith / 15;
                }
            }
        });
        surfaceView.setSurfaceTextureListener(new TextureView.SurfaceTextureListener() {
            @Override
            public void onSurfaceTextureAvailable(SurfaceTexture surface, int width, int height) {

                videoSurface = new Surface(surface);

                if (aliVcMediaPlayer == null) {
                    aliVcMediaPlayer = new AliVcMediaPlayer(DemoLiveAnswerActivity.this, videoSurface);
                    aliVcMediaPlayer.enableNativeLog();

                    aliVcMediaPlayer.setSEIDataListener(new MySeiDataListener(DemoLiveAnswerActivity.this));
                    aliVcMediaPlayer.setFrameInfoListener(new MyFrameInfoListener(DemoLiveAnswerActivity.this));
                    aliVcMediaPlayer.setErrorListener(new MyErrorListener(DemoLiveAnswerActivity.this));

                    aliVcMediaPlayer.setMaxBufferDuration(3 * 1000);
                    aliVcMediaPlayer.prepareAndPlay(liveUrl);
                } else {
                    aliVcMediaPlayer.setVideoSurface(videoSurface);
                }
            }

            @Override
            public void onSurfaceTextureSizeChanged(SurfaceTexture surface, int width, int height) {
                aliVcMediaPlayer.setSurfaceChanged();
            }

            @Override
            public boolean onSurfaceTextureDestroyed(SurfaceTexture surface) {
                return false;
            }

            @Override
            public void onSurfaceTextureUpdated(SurfaceTexture surface) {

            }
        });

    }

    private static class MyErrorListener implements MediaPlayer.MediaPlayerErrorListener {

        private WeakReference<DemoLiveAnswerActivity> answerActivityWeakReference;

        private MyErrorListener(DemoLiveAnswerActivity answerActivity) {
            answerActivityWeakReference = new WeakReference<DemoLiveAnswerActivity>(answerActivity);
        }


        @Override
        public void onError(int i, String s) {
            DemoLiveAnswerActivity answerActivity = answerActivityWeakReference.get();
            if (answerActivity != null) {
                answerActivity.onError(i, s);
            }
        }
    }

    private int retryNum = 0;

    private void onError(int code, String msg) {
        VcPlayerLog.d("DemoLiveAnserActivity", "播放出错：code = " + code);
        if (code == AliyunErrorCode.ALIVC_ERROR_LOADING_TIMEOUT.getCode()) {
            if (retryNum > 5) {
                Toast.makeText(this, "播放失败！", Toast.LENGTH_SHORT).show();
            } else {

                Toast.makeText(this, "重试播放。。", Toast.LENGTH_SHORT).show();
                //尝试重试
                retryNum++;
                if (aliVcMediaPlayer != null) {
                    aliVcMediaPlayer.stop();
                    aliVcMediaPlayer.prepareAndPlay(liveUrl);
                }

            }
        }
    }

    private static class MyFrameInfoListener implements MediaPlayer.MediaPlayerFrameInfoListener {
        private WeakReference<DemoLiveAnswerActivity> answerActivityWeakReference;

        private MyFrameInfoListener(DemoLiveAnswerActivity answerActivity) {
            answerActivityWeakReference = new WeakReference<DemoLiveAnswerActivity>(answerActivity);
        }


        @Override
        public void onFrameInfoListener() {
            DemoLiveAnswerActivity answerActivity = answerActivityWeakReference.get();
            if (answerActivity != null) {
                answerActivity.onFrameInfoListener();
            }
        }
    }

    private void onFrameInfoListener() {
        retryNum = 0;//重置
    }

    private static class MySeiDataListener implements MediaPlayer.MediaPlayerSEIDataListener {

        private WeakReference<DemoLiveAnswerActivity> answerActivityWeakReference;

        private MySeiDataListener(DemoLiveAnswerActivity answerActivity) {
            answerActivityWeakReference = new WeakReference<DemoLiveAnswerActivity>(answerActivity);
        }

        @Override
        public void onSeiUserUnregisteredData(String data) {
            DemoLiveAnswerActivity answerActivity = answerActivityWeakReference.get();
            if (answerActivity != null) {
                answerActivity.onSEI_userUnregisteredData(data);
            }
        }
    }

    private void onSEI_userUnregisteredData(String data) {
        VcPlayerLog.d("DemoLiveAnswerActivity", "onSEI_userUnregisteredData :SEI data = " + data);
        Toast.makeText(getApplicationContext(), "SEI ：" + data, Toast.LENGTH_SHORT).show();
        praseData(data);
    }


    public static class SeiInfo {
        public String questionId;
        public String type;
        public int showTime;

        public static SeiInfo getFromJson(JSONObject jsonObject) {
            if (jsonObject == null) {
                return null;

            }

            SeiInfo seiInfo = new SeiInfo();
            seiInfo.questionId = JsonUtil.getString(jsonObject, "questionId");
            seiInfo.type = JsonUtil.getString(jsonObject, "type");
            seiInfo.showTime = JsonUtil.getInt(jsonObject, "showTime");

            return seiInfo;
        }
    }

    private void praseData(String data) {
        JSONObject jsonObject = null;
        try {
            jsonObject = new JSONObject(data);
        } catch (JSONException e) {
            e.printStackTrace();
        }

        SeiInfo seiInfo = SeiInfo.getFromJson(jsonObject);
        if (seiInfo == null) {
            Toast.makeText(getApplicationContext(), "SEI 数据错误", Toast.LENGTH_SHORT).show();
            return;
        }

        if (seiInfo.type.equals("startAnswer")) {
            requestQuestion(seiInfo.questionId);
        } else if (seiInfo.type.equals("showResult")) {
            requestQuestionReport(seiInfo.questionId, seiInfo.showTime);
        }

    }

    private void requestQuestion(String questionId) {

        GetQuestionRequest.GetQuestionParam getQuestionParam = new GetQuestionRequest.GetQuestionParam();
        getQuestionParam.liveId = liveId;
        getQuestionParam.questionId = questionId;

        GetQuestionRequest getQuestionRequest = new GetQuestionRequest(getApplicationContext(),
                getQuestionParam, new BaseRequest.OnRequestListener<QuestionInfo>() {
            @Override
            public void onSuccess(QuestionInfo questionInfo, String requestId) {

                if (questionInfo != null && questionInfo.remainSeconds > 0) {
                    if (questionInfo.remainSeconds > 10) {
                        questionInfo.remainSeconds = 10;
                    }
                    mLiveAnswerView.setQuestion(questionInfo, canAnswer);
                    mLiveAnswerView.show();
                    miniLiveSurface();
                } else {
                    canAnswer = false;
                    Toast.makeText(getApplicationContext(), "答题超时！", Toast.LENGTH_SHORT).show();
                }
            }

            @Override
            public void onFail(int code, String msg, String requestId) {
                Toast.makeText(getApplicationContext(), "获取题目失败！" + msg, Toast.LENGTH_SHORT).show();
            }
        });
        getQuestionRequest.get();
    }

    private boolean canAnswer = true;
    private String userId = UUID.randomUUID().toString();

    private void sendMyAnswerToServer(AnswerRequestInfo answerRequestInfo) {
        //发送我的答题结果给服务器。
        if (answerRequestInfo == null) {
            canAnswer = false;
            return;
        }

        AnswerRequest.AnswerParam answerParam = new AnswerRequest.AnswerParam();
        answerParam.answerId = answerRequestInfo.answerId;
        answerParam.liveId = liveId;
        answerParam.questionId = answerRequestInfo.questionId;
        answerParam.userId = userId;

        AnswerRequest answerRequest = new AnswerRequest(getApplicationContext(),
                answerParam, new BaseRequest.OnRequestListener<AnswerInfo>() {
            @Override
            public void onSuccess(AnswerInfo requestInfo, String requestId) {
                canAnswer = requestInfo.isCorrect;
            }

            @Override
            public void onFail(int code, String msg, String requestId) {
                Toast.makeText(getApplicationContext(), "答题失败！" + msg, Toast.LENGTH_SHORT).show();
                //判断结果,正确可以继续答题，不正确不能答题
                canAnswer = false;
            }
        });
        answerRequest.get();
    }

    private void miniLiveSurface() {

        minSurfaceAnim.start();

    }

    private void maxLiveSurface() {
        maxSurfaceAnim.start();
    }


    private void requestQuestionReport(String questionId, final int showTime) {

        GetQuestionReportRequest.GetQuestionReportParam getQuestionParam = new GetQuestionReportRequest.GetQuestionReportParam();
        getQuestionParam.liveId = liveId;
        getQuestionParam.questionId = questionId;

        GetQuestionReportRequest reportRequest = new GetQuestionReportRequest(getApplicationContext(),
                getQuestionParam, new BaseRequest.OnRequestListener<QuestionInfo>() {
            @Override
            public void onSuccess(QuestionInfo questionReportInfo, String requestId) {
                int finalShowTime = showTime;
                if (finalShowTime <= 0) {
                    finalShowTime = 5;
                }
                mLiveAnswerView.setResult(questionReportInfo, canAnswer, finalShowTime);
                mLiveAnswerView.show();
                miniLiveSurface();

            }

            @Override
            public void onFail(int code, String msg, String requestId) {
                Toast.makeText(getApplicationContext(), "获取汇总失败！" + msg, Toast.LENGTH_SHORT).show();
            }
        });
        reportRequest.get();
        ;

    }


    @Override
    protected void onDestroy() {
        super.onDestroy();
        aliVcMediaPlayer.stop();
        aliVcMediaPlayer.destroy();
    }
}
