package com.aliyun.vodplayer.demo.activity.advance;

import android.content.pm.ActivityInfo;
import android.os.Build;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.WindowManager;
import android.widget.TextView;
import android.widget.Toast;

import com.aliyun.vodplayer.demo.R;
import com.aliyun.vodplayer.demo.utils.BaseAppCompatActivity;
import com.aliyun.vodplayer.media.AliyunLocalSource;
import com.aliyun.vodplayer.media.AliyunVidSts;
import com.aliyun.vodplayer.media.IAliyunVodPlayer;
import com.aliyun.vodplayerview.widget.AliyunVodPlayerView;

import java.lang.ref.WeakReference;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

public class FixedSkinActivity extends BaseAppCompatActivity {

    private AliyunVodPlayerView mAliyunVodPlayerView;

    private SimpleDateFormat format = new SimpleDateFormat("HH:mm:ss.SS");
    private List<String> logStrs = new ArrayList<>();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

        setContentView(R.layout.activity_skin);

        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        findViewById(R.id.log).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                LayoutInflater inflater = LayoutInflater.from(FixedSkinActivity.this);
                View view = inflater.inflate(R.layout.view_log, null);

                TextView textview = (TextView) view.findViewById(R.id.log);
                if (mAliyunVodPlayerView != null) {
                    for (String log : logStrs) {
                        textview.append("     " + log + "\n");
                    }
                }
                AlertDialog.Builder alertDialog = new AlertDialog.Builder(FixedSkinActivity.this);
                alertDialog.setTitle(R.string.player_log);
                alertDialog.setView(view);
                alertDialog.setPositiveButton(R.string.ok, null);
                AlertDialog alert = alertDialog.create();
                alert.show();
            }
        });
        findViewById(R.id.blue).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                mAliyunVodPlayerView.setTheme(AliyunVodPlayerView.Theme.Blue);
            }
        });
        findViewById(R.id.green).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                mAliyunVodPlayerView.setTheme(AliyunVodPlayerView.Theme.Green);
            }
        });
        findViewById(R.id.orange).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                mAliyunVodPlayerView.setTheme(AliyunVodPlayerView.Theme.Orange);
            }
        });
        findViewById(R.id.red).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                mAliyunVodPlayerView.setTheme(AliyunVodPlayerView.Theme.Red);
                mAliyunVodPlayerView.setTheme(AliyunVodPlayerView.Theme.Red);
            }
        });

        findViewById(R.id.change_source).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                setPlaySource();
            }
        });


        mAliyunVodPlayerView = (AliyunVodPlayerView) findViewById(R.id.video_view);
        mAliyunVodPlayerView.setOnPreparedListener(new MyPrepareListener(this));
        mAliyunVodPlayerView.setOnCompletionListener(new MyCompletionListener(this));
        mAliyunVodPlayerView.setOnFirstFrameStartListener(new MyFirstFrameListener(this));
        mAliyunVodPlayerView.setOnChangeQualityListener(new MyChangeQualityListener(this));
        mAliyunVodPlayerView.setOnStoppedListner(new MyStoppedListener(this));
        mAliyunVodPlayerView.enableNativeLog();

        fixSkin();

        setPlaySource();
    }

    private static class MyPrepareListener implements IAliyunVodPlayer.OnPreparedListener {

        private WeakReference<FixedSkinActivity> activityWeakReference;

        public MyPrepareListener(FixedSkinActivity activity) {
            activityWeakReference = new WeakReference<FixedSkinActivity>(activity);
        }


        @Override
        public void onPrepared() {
            FixedSkinActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onPrepared();
            }
        }
    }

    private void onPrepared() {
        logStrs.add(format.format(new Date()) + getString(R.string.log_prepare_success));
        Toast.makeText(FixedSkinActivity.this.getApplicationContext(), R.string.toast_prepare_success, Toast.LENGTH_SHORT).show();
    }

    private static class MyCompletionListener implements IAliyunVodPlayer.OnCompletionListener {

        private WeakReference<FixedSkinActivity> activityWeakReference;

        public MyCompletionListener(FixedSkinActivity activity) {
            activityWeakReference = new WeakReference<FixedSkinActivity>(activity);
        }

        @Override
        public void onCompletion() {
            FixedSkinActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onCompletion();
            }
        }
    }

    private void onCompletion() {
        logStrs.add(format.format(new Date()) + getString(R.string.log_play_completion));
        Toast.makeText(FixedSkinActivity.this.getApplicationContext(), R.string.toast_play_compleion, Toast.LENGTH_SHORT).show();
    }

    private static class MyFirstFrameListener implements IAliyunVodPlayer.OnFirstFrameStartListener {

        private WeakReference<FixedSkinActivity> activityWeakReference;

        public MyFirstFrameListener(FixedSkinActivity activity) {
            activityWeakReference = new WeakReference<FixedSkinActivity>(activity);
        }

        @Override
        public void onFirstFrameStart() {
            FixedSkinActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onFirstFrameStart();
            }
        }
    }

    private void onFirstFrameStart() {
        Map<String, String> debugInfo = mAliyunVodPlayerView.getAllDebugInfo();
        long createPts = 0;
        if (debugInfo.get("create_player") != null) {
            String time = debugInfo.get("create_player");
            createPts = (long) Double.parseDouble(time);
            logStrs.add(format.format(new Date(createPts)) + getString(R.string.log_player_create_success));
        }
        if (debugInfo.get("open-url") != null) {
            String time = debugInfo.get("open-url");
            long openPts = (long) Double.parseDouble(time) + createPts;
            logStrs.add(format.format(new Date(openPts)) + getString(R.string.log_open_url_success));
        }
        if (debugInfo.get("find-stream") != null) {
            String time = debugInfo.get("find-stream");
            long findPts = (long) Double.parseDouble(time) + createPts;
            logStrs.add(format.format(new Date(findPts)) + getString(R.string.log_request_stream_success));
        }
        if (debugInfo.get("open-stream") != null) {
            String time = debugInfo.get("open-stream");
            long openPts = (long) Double.parseDouble(time) + createPts;
            logStrs.add(format.format(new Date(openPts)) + getString(R.string.log_start_open_stream));
        }
        logStrs.add(format.format(new Date()) + getString(R.string.log_first_frame_played));
    }

    private static class MyChangeQualityListener implements IAliyunVodPlayer.OnChangeQualityListener {

        private WeakReference<FixedSkinActivity> activityWeakReference;

        public MyChangeQualityListener(FixedSkinActivity activity) {
            activityWeakReference = new WeakReference<FixedSkinActivity>(activity);
        }


        @Override
        public void onChangeQualitySuccess(String finalQuality) {
            FixedSkinActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onChangeQualitySuccess(finalQuality);
            }

        }

        @Override
        public void onChangeQualityFail(int code, String msg) {
            FixedSkinActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onChangeQualityFail(code, msg);
            }
        }
    }


    private void onChangeQualitySuccess(String finalQuality) {
        logStrs.add(format.format(new Date()) + getString(R.string.log_change_quality_success));
        Toast.makeText(FixedSkinActivity.this.getApplicationContext(), getString(R.string.log_change_quality_success), Toast.LENGTH_SHORT).show();
    }

    private void onChangeQualityFail(int code, String msg) {
        logStrs.add(format.format(new Date()) + getString(R.string.log_change_quality_fail) + " : " + msg);
        Toast.makeText(FixedSkinActivity.this.getApplicationContext(), getString(R.string.log_change_quality_fail), Toast.LENGTH_SHORT).show();
    }


    private static class MyStoppedListener implements IAliyunVodPlayer.OnStoppedListener {

        private WeakReference<FixedSkinActivity> activityWeakReference;

        public MyStoppedListener(FixedSkinActivity activity) {
            activityWeakReference = new WeakReference<FixedSkinActivity>(activity);
        }

        @Override
        public void onStopped() {
            FixedSkinActivity activity = activityWeakReference.get();
            if (activity != null) {
                activity.onStopped();
            }
        }
    }

    private void onStopped() {
        Toast.makeText(FixedSkinActivity.this.getApplicationContext(), R.string.log_play_stopped, Toast.LENGTH_SHORT).show();
    }

    private void fixSkin() {
        mAliyunVodPlayerView.setLockPortraitMode(new IAliyunVodPlayer.LockPortraitListener() {
            @Override
            public void onLockScreenMode(int screenMode) {
                if (screenMode == 1) //跳到小屏
                {
                    if (Build.DEVICE.equalsIgnoreCase("mx5")
                            || Build.DEVICE.equalsIgnoreCase("Redmi Note2")
                            || Build.DEVICE.equalsIgnoreCase("Z00A_1")) {
                        getSupportActionBar().show();
                    }
                    FixedSkinActivity.this.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
                } else {
                    if (Build.DEVICE.equalsIgnoreCase("mx5")
                            || Build.DEVICE.equalsIgnoreCase("Redmi Note2")
                            || Build.DEVICE.equalsIgnoreCase("Z00A_1")) {
                        getSupportActionBar().hide();
                    } else if (!(Build.DEVICE.equalsIgnoreCase("V4") && Build.MANUFACTURER.equalsIgnoreCase("Meitu"))) {
                        FixedSkinActivity.this.getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
                        mAliyunVodPlayerView.setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                                | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                                | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                                | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                                | View.SYSTEM_UI_FLAG_FULLSCREEN
                                | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY);
                    }
                }
            }
        });
    }

    private void setPlaySource() {


        String type = getIntent().getStringExtra("type");
        if (TextUtils.isEmpty(type)) {
            return;
        }

        if (type.equals("localSource")) {

            AliyunLocalSource.AliyunLocalSourceBuilder alsb = new AliyunLocalSource.AliyunLocalSourceBuilder();
            alsb.setSource(getIntent().getStringExtra("url"));
            AliyunLocalSource localSource = alsb.build();
            mAliyunVodPlayerView.setLocalSource(localSource);

        } else if (type.equals("vidsts")) {

            String vid = getIntent().getStringExtra("vid");
            String akId = getIntent().getStringExtra("akId");
            String akSecret = getIntent().getStringExtra("akSecret");
            String scuToken = getIntent().getStringExtra("scuToken");

            AliyunVidSts vidSts = new AliyunVidSts();
            vidSts.setVid(vid);
            vidSts.setAcId(akId);
            vidSts.setAkSceret(akSecret);
            vidSts.setSecurityToken(scuToken);
            mAliyunVodPlayerView.setVidSts(vidSts);
        }

    }

    @Override
    protected void onResume() {
        super.onResume();

        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

        if (mAliyunVodPlayerView != null) {
            mAliyunVodPlayerView.onResume();
        }

    }


    @Override
    protected void onStop() {
        super.onStop();

        if (mAliyunVodPlayerView != null) {
            mAliyunVodPlayerView.onStop();
        }
    }

    @Override
    protected void onDestroy() {
        if (mAliyunVodPlayerView != null) {
            mAliyunVodPlayerView.onDestroy();
        }

        super.onDestroy();

    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (mAliyunVodPlayerView != null) {
            boolean handler = mAliyunVodPlayerView.onKeyDown(keyCode, event);
            if (!handler) {
                return false;
            }
        }
        return super.onKeyDown(keyCode, event);
    }


}
