package com.aliyun.vodplayer.demo.activity.advance.liveanswer;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.RectF;
import android.support.annotation.Nullable;
import android.util.AttributeSet;
import android.view.View;

import com.aliyun.vodplayer.demo.R;

/*
 * Copyright (C) 2010-2017 Alibaba Group Holding Limited.
 */
public class AnswerProgressBar extends View {

    private Type mType = Type.Gray;
    private int mMax = 100;
    private int mProgress = 0;

    private Paint bkgPaint;
    private Paint progressPaint;
    private float strokeWidth;

    public AnswerProgressBar(Context context) {
        super(context);
        init();
    }

    public AnswerProgressBar(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    public AnswerProgressBar(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    private void init() {
        strokeWidth = getResources().getDimension(R.dimen.answer_progressbar_stroke_width);

        bkgPaint = new Paint();
        bkgPaint.setAntiAlias(true);// 设置画笔的锯齿效果

        progressPaint = new Paint();
        progressPaint.setAntiAlias(true);// 设置画笔的锯齿效果
    }


    public void setMax(int max) {
        mMax = max;
        invalidate();
    }

    public void setProgress(int progress) {
        mProgress = progress;
        invalidate();
    }

    public static enum Type {
        Gray, Red, Blue
    }

    public void setType(Type type) {
        mType = type;
        invalidate();
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);

        drawBkg(canvas);
        drawProgress(canvas);
    }

    private void drawBkg(Canvas canvas) {
        //画圆角矩形
        bkgPaint.setStyle(Paint.Style.FILL);//充满
        bkgPaint.setColor(Color.parseColor("#EAEAEA"));
        RectF oval3 = new RectF(getPaddingLeft() + strokeWidth, getPaddingTop() + strokeWidth, getWidth() - getPaddingRight() - strokeWidth, getHeight() - getPaddingBottom() - strokeWidth);// 设置个新的长方形
        canvas.drawRoundRect(oval3, getHeight() / 2, getHeight() / 2, bkgPaint);//第二个参数是x半径，第三个参数是y半径

        bkgPaint.setStyle(Paint.Style.STROKE);//边框
        bkgPaint.setColor(Color.parseColor("#AAAAAA"));
        bkgPaint.setStrokeWidth(getResources().getDimension(R.dimen.answer_progressbar_stroke_width));
        canvas.drawRoundRect(oval3, getHeight() / 2, getHeight() / 2, bkgPaint);//第二个参数是x半径，第三个参数是y半径
    }


    private void drawProgress(Canvas canvas) {
        if (mProgress <= 0 || mMax <= 0) {
            return;
        }

        //画圆角矩形
        progressPaint.setStyle(Paint.Style.FILL);//充满
        String progressColor = "#858585";
        if (mType == Type.Blue) {
            progressColor = "#4EA7C0";
        } else if (mType == Type.Red) {
            progressColor = "#E94033";
        } else {
            progressColor = "#858585";
        }

        int rectWidth = (int) (mProgress * (getWidth() - getHeight()) * 1.0f / mMax) + getHeight();

        progressPaint.setColor(Color.parseColor(progressColor));
        RectF oval3 = new RectF(getPaddingLeft() + strokeWidth, getPaddingTop() + strokeWidth, rectWidth + getPaddingLeft() - strokeWidth, getHeight() - getPaddingBottom() - strokeWidth);// 设置个新的长方形
        canvas.drawRoundRect(oval3, getHeight() / 2, getHeight() / 2, progressPaint);//第二个参数是x半径，第三个参数是y半径

        progressPaint.setStyle(Paint.Style.STROKE);//边框
        progressPaint.setColor(Color.parseColor("#AAAAAA"));
        progressPaint.setStrokeWidth(strokeWidth);
        canvas.drawRoundRect(oval3, getHeight() / 2, getHeight() / 2, progressPaint);//第二个参数是x半径，第三个参数是y半径
    }


}
