package com.aliyun.vodplayer.demo.activity.advance.liveanswer;

import android.content.Context;
import android.graphics.Color;
import android.os.Handler;
import android.os.Message;
import android.support.annotation.Nullable;
import android.util.AttributeSet;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.aliyun.vodplayer.demo.R;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.bean.AnswerRequestInfo;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.bean.OptionInfo;
import com.aliyun.vodplayer.demo.activity.advance.liveanswer.bean.QuestionInfo;

import java.lang.ref.WeakReference;

/*
 * Copyright (C) 2010-2017 Alibaba Group Holding Limited.
 */
public class LiveAnswerView extends LinearLayout {

    private Animation popInAnim;
    private Animation popOutAnim;

    private TextView mResultView;
    private TextView mQuestionView;
    private LinearLayout mAnswerList;

    private QuestionInfo mQuestionInfo = null;
    private OptionInfo mMyAnswer = null;

    private boolean mCanAnswer = true;

    private DecountTimer decountTimer;

    public LiveAnswerView(Context context) {
        super(context);
        init();
    }


    public LiveAnswerView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        init();
    }


    public LiveAnswerView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    private void init() {

        decountTimer = new DecountTimer(this);
        popInAnim = AnimationUtils.loadAnimation(getContext().getApplicationContext(), R.anim.pop_in);

        popOutAnim = AnimationUtils.loadAnimation(getContext().getApplicationContext(), R.anim.pop_out);
        popOutAnim.setAnimationListener(new Animation.AnimationListener() {
            @Override
            public void onAnimationStart(Animation animation) {

            }

            @Override
            public void onAnimationEnd(Animation animation) {
                setVisibility(INVISIBLE);
            }

            @Override
            public void onAnimationRepeat(Animation animation) {

            }
        });

        setOrientation(VERTICAL);
        setBackgroundResource(R.drawable.live_answer_bkg);
        setGravity(Gravity.CENTER_HORIZONTAL);
        int paddingDemin = getResources().getDimensionPixelSize(R.dimen.live_answer_padding);
        setPadding(paddingDemin, paddingDemin, paddingDemin, paddingDemin);

        LayoutInflater.from(getContext()).inflate(R.layout.view_live_answer, this, true);

        mResultView = (TextView) findViewById(R.id.result);
        mQuestionView = (TextView) findViewById(R.id.question);
        mAnswerList = (LinearLayout) findViewById(R.id.answer_list);

        setVisibility(INVISIBLE);

        setKeepScreenOn(true);

    }

    public void show() {
        setVisibility(VISIBLE);
        startAnimation(popInAnim);
    }


    public void hide() {
        startAnimation(popOutAnim);
    }

    public void setQuestion(QuestionInfo question, boolean canAnswer) {
        if (question == null) {
            return;
        }

        mMyAnswer = null;
        bindQuestionToView(question, canAnswer);
        startDecount();
    }

    private boolean hasLose = false;

    private void bindQuestionToView(QuestionInfo question, boolean canAnswer) {
        if (mCanAnswer) {
            hasLose = false;
        } else {
            hasLose = true;
        }
        mCanAnswer = canAnswer;
        mQuestionInfo = question;
        mQuestionView.setText(question.questionTitle);
        Log.d("lfj0118", "canAnswer = " + mCanAnswer);
        if (mCanAnswer) {
            mResultView.setText("已开始答题");
            mResultView.setTextColor(Color.parseColor("#4F504F"));
        } else {
            mResultView.setText("已淘汰");
            mResultView.setTextColor(Color.parseColor("#F5A622"));
        }

        count = question.remainSeconds - 1;

        addAllAnswerItems(question.options);
    }

    private int count = 9;
    private boolean countEnd = false;

    private static class DecountTimer extends Handler{

        private WeakReference<LiveAnswerView> liveAnswerViewWeakReference;

       public DecountTimer (LiveAnswerView view){
           liveAnswerViewWeakReference = new WeakReference<LiveAnswerView>(view);
        }

        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);

            LiveAnswerView liveAnswerView = liveAnswerViewWeakReference.get();
            if(liveAnswerView != null){
                liveAnswerView.handleMessage(msg);
            }
        }
    }


        private void handleMessage(Message msg) {

            if (WHAT_ANSWER_DECOUNT == msg.what) {
                if (mCanAnswer) {
                    mResultView.setText("" + count);
                    mResultView.setTextColor(Color.parseColor("#4F504F"));
                } else {
                    mResultView.setText("已淘汰");
                    mResultView.setTextColor(Color.parseColor("#F5A622"));
                }

                if (count > 0) {
                    count--;
                    decountTimer.sendEmptyMessageDelayed(WHAT_ANSWER_DECOUNT, 1000);
                } else {
                    //倒计时结束
                    countEnd = true;
                    if (onTimeRemainListener != null) {
                        onTimeRemainListener.decountEnd();
                    }
                }
            } else if (WHAT_RESULT_DECOUNT == msg.what) {
                if (onTimeRemainListener != null) {
                    onTimeRemainListener.resultEnd();
                }
            }
        }


    int WHAT_ANSWER_DECOUNT = 0;
    int WHAT_RESULT_DECOUNT = 1;

    private void startDecount() {
        countEnd = false;
        decountTimer.sendEmptyMessageDelayed(WHAT_ANSWER_DECOUNT, 1000);
    }

    private void addAllAnswerItems(OptionInfo[] answers) {
        mAnswerList.removeAllViews();

        if (answers == null) {
            return;
        }

        for (OptionInfo ans : answers) {
            final AnswerItemView itemView = new AnswerItemView(getContext());
            itemView.setAnswer(ans);
            itemView.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (mMyAnswer == null && !countEnd && !hasLose && mCanAnswer) {
                        itemView.chooseThis();
                        mMyAnswer = itemView.getAnswer();

                        if (onTimeRemainListener != null) {
                            AnswerRequestInfo answerRequestInfo = getMyAnswer();
                            onTimeRemainListener.onChoose(answerRequestInfo);
                        }
                    }
                }
            });

            mAnswerList.addView(itemView, RelativeLayout.LayoutParams.MATCH_PARENT, RelativeLayout.LayoutParams.WRAP_CONTENT);
        }
    }

    public void setResult(QuestionInfo questionReportInfo, boolean canAnswer, int showTimeSec) {

        countEnd = true;

        if (questionReportInfo == null) {
            return;
        }

        mCanAnswer = canAnswer;
        mQuestionInfo = questionReportInfo;

        mQuestionView.setText(mQuestionInfo.questionTitle);
        if (canAnswer) {
            mResultView.setText("答对了");
            mResultView.setTextColor(Color.parseColor("#48A6C0"));
        } else {
            if (hasLose) {
                mResultView.setText("已淘汰");
                mResultView.setTextColor(Color.parseColor("#F5A622"));
            } else {
                mResultView.setText("答错了");
                mResultView.setTextColor(Color.parseColor("#EC5163"));
            }
        }

        int childCount = mAnswerList.getChildCount();

        if (childCount == 0) {
            //答题过了，才进来。。
            addAllAnswerItems(questionReportInfo.options);
        }

        childCount = mAnswerList.getChildCount();
        if (childCount > 0) {
            for (int i = 0; i < childCount; i++) {
                ((AnswerItemView) mAnswerList.getChildAt(i)).showResult(questionReportInfo);
            }
        }

        decountTimer.sendEmptyMessageDelayed(WHAT_RESULT_DECOUNT, showTimeSec * 1000);

    }


    public AnswerRequestInfo getMyAnswer() {
        if (mMyAnswer == null || mQuestionInfo == null) {
            return null;
        }

        AnswerRequestInfo answerRequestInfo = new AnswerRequestInfo();
        answerRequestInfo.answerId = mMyAnswer.answerId;
        answerRequestInfo.questionId = mQuestionInfo.questionId;

        return answerRequestInfo;
    }

    public interface OnTimeRemainListener {
        void decountEnd();

        void resultEnd();

        void onChoose(AnswerRequestInfo answerRequestInfo);
    }

    private OnTimeRemainListener onTimeRemainListener;

    public void setOnTimeRemainListener(OnTimeRemainListener l) {
        onTimeRemainListener = l;
    }

}
